package seu;

import java.io.IOException;

import rescuecore2.Constants;
import rescuecore2.components.ComponentConnectionException;
import rescuecore2.components.ComponentLauncher;
import rescuecore2.components.TCPComponentLauncher;
import rescuecore2.config.Config;
import rescuecore2.config.ConfigException;
import rescuecore2.connection.ConnectionException;
import rescuecore2.misc.CommandLineOptions;
import rescuecore2.registry.Registry;
import rescuecore2.standard.entities.StandardEntityFactory;
import rescuecore2.standard.entities.StandardPropertyFactory;
import rescuecore2.standard.messages.StandardMessageFactory;
import seu.agent.Ambulance.SEUAmbulanceCentre;
import seu.agent.Ambulance.SEUAmbulanceTeam;
import seu.agent.Firebrigade.SEUFireBrigade;
import seu.agent.Firebrigade.SEUFireStation;
import seu.agent.Police.SEUPoliceForce;
import seu.agent.Police.SEUPoliceOffice;

/**
 * Launch SEU's Agents
 */
public final class Main {

	private static final String RANDOM_SEED = "122";

	// Rule 中规定的智能体最大数量
	private static final int maxAgent = 30;
	private static final int maxCentre = 5;

	private Main() {
	}

	/**
	 * @param 启动参数的定义如下
	 *            -p <port>, -h <hostname>, -fb <fire brigades>, -pf <police
	 *            forces>, -at <ambulance teams>
	 */
	public static void main(String[] args) {

		try {
			Registry.SYSTEM_REGISTRY
					.registerEntityFactory(StandardEntityFactory.INSTANCE);
			Registry.SYSTEM_REGISTRY
					.registerMessageFactory(StandardMessageFactory.INSTANCE);
			Registry.SYSTEM_REGISTRY
					.registerPropertyFactory(StandardPropertyFactory.INSTANCE);
			Config config = new Config();

			args = CommandLineOptions.processArgs(args, config);

			// Keep
			config.setValue(Constants.RANDOM_SEED_KEY, RANDOM_SEED);

			int port = config.getIntValue(Constants.KERNEL_PORT_NUMBER_KEY,
					Constants.DEFAULT_KERNEL_PORT_NUMBER);

			String defaultHost = config.getValue(
					Constants.KERNEL_HOST_NAME_KEY,
					Constants.DEFAULT_KERNEL_HOST_NAME);

			int fb = -1;
			int fs = -1;

			int pf = -1;
			int po = -1;

			int at = -1;
			int ac = -1;

			fb = (args[0].equals("-1")) ? maxAgent : Integer.parseInt(args[0]);

			fs = (args[1].equals("-1")) ? maxCentre : Integer.parseInt(args[1]);

			pf = (args[2].equals("-1")) ? maxAgent : Integer.parseInt(args[2]);

			po = (args[3].equals("-1")) ? maxCentre : Integer.parseInt(args[3]);

			at = (args[4].equals("-1")) ? maxAgent : Integer.parseInt(args[4]);

			ac = (args[5].equals("-1")) ? maxCentre : Integer.parseInt(args[5]);

			String host = (args[6].equals("-1")) ? defaultHost : args[6];

			ComponentLauncher launcher = new TCPComponentLauncher(host, port,
					config);

			// 建立连接
			connect(launcher, fb, fs, pf, po, at, ac, config);

		} catch (IOException e) {
			e.printStackTrace();
		} catch (ConfigException e) {
			e.printStackTrace();
		} catch (ConnectionException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private static void connect(ComponentLauncher launcher, int fb, int fs,
			int pf, int po, int at, int ac, Config config)
			throws InterruptedException, ConnectionException {

		int i = 0;
		try {
			while (fb-- > 0) {
				launcher.connect(new SEUFireBrigade());
				System.out.println("Connecting Fire Brigade " + (i++)
						+ "... Success");
			}
		} catch (ComponentConnectionException e) {
			System.out.println("Fire Brigade Conncetion Done");
		}

		i = 0;
		try {
			while (fs-- > 0) {
				launcher.connect(new SEUFireStation());
				System.out.println("Connecting FireStation " + (i++)
						+ "... Success");
			}
		} catch (ComponentConnectionException e) {
			System.out.println("Fire Station Connection Done");
		}

		i = 0;
		try {
			while (pf-- > 0) {
				launcher.connect(new SEUPoliceForce());
				System.out.println("Connecting Police Force " + (i++)
						+ "... Success");
			}
		} catch (ComponentConnectionException e) {
			System.out.println("Police Force Connection Done");
		}

		i = 0;
		try {
			while (po-- > 0) {
				launcher.connect(new SEUPoliceOffice());
				System.out.println("Connecting PoliceOffice " + (i++)
						+ "... Success");
			}
		} catch (ComponentConnectionException e) {
			System.out.println("Police Office Connection Done");
		}

		i = 0;
		try {
			while (at-- > 0) {
				launcher.connect(new SEUAmbulanceTeam());
				System.out.println("Connecting Ambulance Team " + (i++)
						+ "... Success");
			}
		} catch (ComponentConnectionException e) {
			System.out.println("Ambulance Team Connection Done");
		}

		i = 0;
		try {
			while (ac-- > 0) {
				launcher.connect(new SEUAmbulanceCentre());
				System.out.println("Connecting Ambulance Centre " + (i++)
						+ "... Success");
			}
		} catch (Exception e) {
			System.out.println("Ambulance Centre Connection Done");
			System.out.println("SEU ALL Agents Connection Done");
		}
	}
}
