package seu.agent.Ambulance;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.DecisionManager;
import seu.agent.SEUAgent;
import seu.agent.BasicTask.BasicTask;
import seu.agent.BasicTask.ExplorationTask;
import seu.agent.BasicTask.IsStuckOrBuriedCallHelpTask;
import seu.agent.BasicTask.PartitionExplorationTask;
import seu.agent.BasicTask.RandomWalkTask;
import seu.agent.BasicTask.SaveCivAndAgentTask;
import seu.agent.BasicTask.SaveHuman;
import seu.agent.BasicTask.SaveMyselfTask;
import seu.message.MessagePriority;
import seu.message.Type.AgentIsBuriedMessage;
import seu.message.Type.BuildingIsExploredMessage;
import seu.message.Type.CivilianInformationMessage;
import seu.message.Type.CivilianIsSavedOrDeadMessage;
import seu.message.Type.Message;
import seu.message.Type.MessageCount;
import seu.message.Type.MessageType;

/**
 * SEU's Ambulance Team Agent.
 */
public class SEUAmbulanceTeam extends SEUAgent<AmbulanceTeam> {

	private SaveCivAndAgentTask saveCivilianTasks;
	private SaveCivAndAgentTask saveAgentTask;

	public SEUAmbulanceTeam() {
		super();
	}

	@Override
	public String toString() {
		return "SEU Ambulance Team #" + getNo();
	}

	@Override
	protected void postConnect() {
		super.postConnect();
		new HashSet<EntityID>();

	}

	@Override
	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.AMBULANCE_TEAM);
	}

	protected void processMessage(MessageCount message) {
		int counter = message.getCounter();
		MessageCount reply = new MessageCount(counter + 1);
		sendMessage(reply, MessagePriority.High);
	}

	@Override
	protected void thinkAndAct() {
	}

	protected void printKnownCivilians() {
		for (StandardEntity next : world.getHumans()) {
			if (next instanceof Civilian) {
				Civilian civilian = (Civilian) next;
				EntityID id = civilian.getPosition();
				StandardEntity position = world.getEntity(id);
				if (position instanceof Building) {
				}
			}
		}
	}

	@Override
	protected void processMessage(Message message) {
		if (message instanceof BuildingIsExploredMessage) {
			BuildingIsExploredMessage explorationMessage = (BuildingIsExploredMessage) message;
			processExplorationMessage(explorationMessage);
		}
		if (message instanceof CivilianInformationMessage) {
			CivilianInformationMessage civilianInformationMessage = (CivilianInformationMessage) message;
			processCivilianInformationMessage(civilianInformationMessage);
		}
		if (message instanceof CivilianIsSavedOrDeadMessage) {
			CivilianIsSavedOrDeadMessage civilianIsSavedMessage = (CivilianIsSavedOrDeadMessage) message;
			processCivilianIsSavedMessage(civilianIsSavedMessage);
		}
		if (message instanceof AgentIsBuriedMessage) {
			AgentIsBuriedMessage agentIsBuriedMessage = (AgentIsBuriedMessage) message;
			processAgentIsBuriedMessage(agentIsBuriedMessage);
		}
	}

	protected void processCivilianInformationMessage(
			CivilianInformationMessage message) {
		EntityID location = new EntityID(message.getLocationId());
		StandardEntity civilianLocation = world.getEntity(location);
		Civilian civilian = new Civilian(new EntityID(message.getCivilianId()));
		civilian.setHP(message.getHp());
		civilian.setDamage(message.getDamage());
		civilian.setBuriedness(message.getBuriedness());

		int deathTime = getTimeStep()
				+ DecisionManager.getDeathTime(civilian, timeStep);
		SaveHuman saveCivilianTask = new SaveHuman(civilian, civilianLocation,
				deathTime);
		if (saveCivilianTasks != null) {
			saveCivilianTasks.addSaveTask(saveCivilianTask);
		}
	}

	protected void processAgentIsBuriedMessage(AgentIsBuriedMessage message) {
		EntityID location = new EntityID(message.getLocationId());
		StandardEntity agentLocation = world.getEntity(location);
		Human agent;

		agent = world
				.getEntity(new EntityID(message.getAgentId()), Human.class);
		if (agent != null) {
			agent.setHP(message.getHp());
			agent.setDamage(message.getDamage());
			agent.setBuriedness(message.getBuriedness());
			int deathTime = getTimeStep()
					+ DecisionManager.getDeathTime(agent, timeStep);
			SaveHuman saveHumanTask = new SaveHuman(agent, agentLocation,
					deathTime);

			int dis = world.getDistance(me().getID(), location);

			if (saveAgentTask != null && dis < 100000 && dis > 0
			// 救在100米范围内的智能体
			// && (isInMyPartition(agentLocation) ||
			// isInNeighborRegion(agentLocation))
			) {
				saveAgentTask.addSaveTask(saveHumanTask);
			}
		}
	}

	public void processCivilianIsSavedMessage(
			CivilianIsSavedOrDeadMessage message) {
		EntityID civilianId = message.getCivilianId();
		if (saveCivilianTasks != null) {

			saveCivilianTasks.removeSaveTask(civilianId);
		}
	}

	@Override
	protected boolean canRescue() {
		return true;
	}

	@SuppressWarnings({ "rawtypes" })
	@Override
	public List<BasicTask> getAgentTasks() {
		List<BasicTask> tasks;

		tasks = new ArrayList<BasicTask>();
		tasks.add(new IsStuckOrBuriedCallHelpTask(this));
		tasks.add(new SaveMyselfTask(this));

		saveAgentTask = new SaveCivAndAgentTask(this, true);
		tasks.add(saveAgentTask);

		saveCivilianTasks = new SaveCivAndAgentTask(this, false);
		tasks.add(saveCivilianTasks);

		tasks.add(new ExplorationTask(this));

		tasks.add(new PartitionExplorationTask(this));

		tasks.add(new RandomWalkTask(this));

		return tasks;
	}

	@Override
	public List<MessageType> getMessagesToListen() {
		List<MessageType> types;

		types = new ArrayList<MessageType>();
		types.add(MessageType.AgentIsBuriedMessage);
		types.add(MessageType.CivilianInformationMessage);
		types.add(MessageType.CivilianIsSavedOrDeadMessage);
		types.add(MessageType.BuildingIsExploredMessage);
		types.add(MessageType.BuildingIsBurningMessage);
		types.add(MessageType.BuildingIsExtinguishedMessage);
		return types;
	}

}
