package seu.agent.BasicTask;

import java.awt.Point;

import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import seu.agent.SEUAgent;
import seu.agent.Police.SEUPoliceForce;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;

public class ClearBlockedRoadTask extends BasicTask<SEUPoliceForce> {

	protected Path path;
	protected Blockade target;
	protected int maxClearDistance;

	public ClearBlockedRoadTask(SEUPoliceForce agent, int maxClearDistance) {
		super(agent);
		this.maxClearDistance = maxClearDistance;
	}

	@Override
	public boolean shouldExecute() {
		State expectedMove;

		expectedMove = getExpectedMove();
		state = expectedMove;
		if (expectedMove == State.GettingCloserToBlock) {
			Path path;

			path = agent.getPathToBlock(target);
			if (path == null || !path.isPassable()) {
				// LOG.debug("Couldn't plan a path to blockade: "
				// + target.getFullDescription());
				state = State.Inactive;
				return false;
			} else {
				// LOG.debug("Getting closer to blockade: " + target
				// + " with path: " + path);
				this.path = path;
				return true;
			}
		} else if (expectedMove == State.ClearingCurrentRoad) {
			return true;
		} else if (expectedMove == State.MovingToBlockedRoad) {
			Path path;

			path = agent.getPathToClosestBlockedRoad();

			if (path == null || !path.isPassable()) {
				state = State.Inactive;
				return false;
			} else {
				this.path = path;
				return true;
			}
		} else if (expectedMove == State.Inactive) {
			// LOG.debug("Nothing to do");
			return false;
		} else {
			// LOG.warn("Invalid expected state" + expectedMove);
			return false;
		}
	}

	@Override
	public void execute() {
		if (state == State.GettingCloserToBlock
				|| state == State.MovingToBlockedRoad) {
			agent.sendMove(path);
		} else if (state == State.ClearingCurrentRoad) {
			agent.sendClear(target.getID());
		} else {
		}
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
		path = null;
	}

	public SEUAgent<PoliceForce> getPoliceAgent() {
		return agent;
	}

	protected State getExpectedMove() {
		Human human;
		Blockade blockade;
		double distance;
		Point agentPoint;

		human = agent.getMeAsHuman();
		blockade = agent.getClosestBlockade();
		if (blockade == null) {
			return State.Inactive;
		}
		target = blockade;
		agentPoint = PositionLocate.getPosition(human, getModel());
		distance = DistanceUtilities.getDistanceToBlock(blockade, agentPoint);
		if (distance > maxClearDistance) {
			return State.GettingCloserToBlock;
		} else {
			return State.ClearingCurrentRoad;
		}
	}
}
