package seu.agent.BasicTask;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.SEUPoliceForce;
import seu.message.Type.ClearPathReason;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class ClearBlockedRoadToRefugeTask extends BasicTask<SEUPoliceForce> {
	private final Comparator<ClearPath> comparator;
	private final PriorityQueue<ClearPath> clearPathTaskQueue;
	private ClearPath currentClearPathTask;

	Point lastPosition;
	Point currentPosition;

	Path path;
	Blockade target;

	int maxDistance;

	public ClearBlockedRoadToRefugeTask(SEUPoliceForce agent, int maxDistance) {
		super(agent);
		comparator = new ClearPathTaskComparator();
		clearPathTaskQueue = new PriorityQueue<ClearPath>(10, comparator);

		this.maxDistance = maxDistance;
		assignRefuge();
		addClearPathToRefugeTask();
	}

	public void assignRefuge() {
		HashSet<StandardEntity> temp;
		temp = (HashSet<StandardEntity>) agent.getWorldModel()
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE);
		Iterator<StandardEntity> it = temp.iterator();
		agent.numOfPF = temp.size();
		while (it.hasNext()) {
			StandardEntity t = (StandardEntity) it.next();
			agent.PF.add(t);
			agent.PFSort.add(t.getID().getValue());
		}
		Collections.sort(agent.PFSort, new IntValueComprator());
		temp = (HashSet<StandardEntity>) agent.getWorldModel()
				.getEntitiesOfType(StandardEntityURN.REFUGE);
		agent.numOfRefuge = temp.size();
		it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = (StandardEntity) it.next();
			agent.refuge.add(t);
			agent.refugeSort.add(t.getID().getValue());
		}
		Collections.sort(agent.PFSort, new IntValueComprator());
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		State expectedMove;
		ClearPath task;
		Human human;

		updateCurrentClearPathTask();
		task = currentClearPathTask;
		currentPosition = PositionLocate.getPosition(agent.getID(), getModel());
		if (agent.targetRefuge == null) {
			return false;
		}
		if (agent.existPO && !agent.isSelected) {
			ClearPath clearRefugePathTask = new ClearPath(
					agent.targetRefuge.getID(), agent.location().getID(),
					ClearPathReason.RefugeIsBlocked);
			agent.isSelected = true;
			// System.out.println(agent.getID().getValue() +
			// "is Responsible for "
			// + agent.targetRefuge.getID().getValue() + "time"
			// + agent.getTimeStep());
			addClearPathTask(clearRefugePathTask);
		}
		if (agent.isSelected == false) {
			return false;
		}
		if (agent.isDone) {
			return false;
		}
		if (agent.getMeAsHuman().getPosition().getValue() == agent.targetRefuge
				.getID().getValue()) {
			agent.isDone = true;
			// System.out.println(agent.getID() + "isDone" +
			// agent.getTimeStep());
			return false;

		}
		expectedMove = getExpectedMove(task);
		state = expectedMove;
		human = agent.getMeAsHuman();

		if (expectedMove == State.ClearingAllBlocksAtLocation) {
			return true;
		} else if (expectedMove == State.MovingToBlockAtCurrentLocation) {
			Path movePath = new Path(agent.location().getID());
			Point targetPoint = new Point(target.getX(), target.getY());
			movePath.setTarget(targetPoint);
			path = movePath;
			return true;
		} else if (expectedMove == State.GoingToGoalToClearPath) {
			Path movePath;
			EntityID id;
			Area destination;

			id = task.getRoadIdFinish();
			destination = getModel().getEntity(id, Area.class);
			movePath = getSearch().getPath(human, destination,
					PathType.Shortest);
			path = movePath;
			return true;
		} else if (expectedMove == State.GoingToStartToClearPath) {
			Path movePath;
			EntityID id;
			Area destination;

			id = task.getRoadIdStart();
			destination = getModel().getEntity(id, Area.class);
			movePath = getSearch().getPath(human, destination,
					PathType.Shortest);
			path = movePath;
			return true;
		} else if (expectedMove == State.GettingCloserToBlock) {
			Path path;

			path = agent.getPathToBlock(target);
			if (path == null || !path.isPassable()) {
				return false;
			} else {
				this.path = path;
				return true;
			}
		} else if (expectedMove == State.ClearingBlocksToPath) {
			return true;
		} else if (expectedMove == State.ClearingPath) {
			return true;
		} else if (expectedMove == State.ClearingAnnoyingBlock) {
			return true;
		} else {
			state = State.Inactive;
			return false;
		}
	}

	@Override
	public void execute() {
		if (state == State.GoingToGoalToClearPath
				|| state == State.GoingToStartToClearPath
				|| state == State.GettingCloserToBlock
				|| state == State.MovingToBlockAtCurrentLocation) {
			agent.sendMove(path);
		} else if (state == State.ClearingBlocksToPath
				|| state == State.ClearingPath
				|| state == State.ClearingAnnoyingBlock
				|| state == State.ClearingAllBlocksAtLocation) {
			agent.sendClear(target.getID());
		}
		lastPosition = currentPosition;
	}

	private void updateCurrentClearPathTask() {
		if (currentClearPathTask == null) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			}
		} else if (isOnStartLocation() && currentClearPathTask.isCompleted()) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			} else {
				currentClearPathTask = null;
			}
		} else if (isOnGoalLocation()) {
			currentClearPathTask.setHasBeenAtStart(true);
		}
	}

	protected State getExpectedMove(ClearPath task) {
		Path path;
		Area start, end;
		EntityID startId, endId;
		Human human;
		boolean onTargetPath;
		State expectedMove;

		if (task == null) {
			return null;
		}
		human = agent.getMeAsHuman();
		// 无法移动
		if (state == State.ClearingAnnoyingBlock) {
			if (target != null) {
				if (RoadUtilities.doesBlockExists(target, getModel())) {
					return State.ClearingAnnoyingBlock;
				} else {
				}
			}
		}
		if (moveFailed()) {
			Blockade closest;
			closest = agent.getClosestBlockade();
			if (closest == null) {
				return null;
			} else {
				target = closest;
				return State.ClearingAnnoyingBlock;
			}
		}

		if (isOnGoalLocation() || isOnStartLocation()) {
			Blockade closest;
			closest = agent.getClosestBlockadeInCurrentLocation();
			if (closest == null) {
				if (isOnStartLocation()) {
					currentClearPathTask.setIsCompleted(true);
				}
			} else {
				target = closest;
				Point agentPoint;
				double d;
				agentPoint = PositionLocate.getPosition(agent.getID(),
						getModel());
				d = DistanceUtilities.getDistanceToBlock(target, agentPoint);
				if (d < maxDistance) {
					return State.ClearingAllBlocksAtLocation;
				} else {
					return State.MovingToBlockAtCurrentLocation;
				}

			}
		}
		if (task.hasBeenAtStart()) {
			startId = task.getRoadIdStart();
			start = getModel().getEntity(startId, Area.class);
			path = getSearch().getPath(human, start, PathType.Shortest);
			onTargetPath = true;
		} else {
			endId = task.getRoadIdFinish();
			end = getModel().getEntity(endId, Area.class);
			path = getSearch().getPath(human, end, PathType.Shortest);
			onTargetPath = false;
		}
		expectedMove = getExpectedMove(path, onTargetPath);
		return expectedMove;
	}

	protected State getExpectedMove(Path path, boolean onTargetPath) {
		if (path == null) {
			return null;
		} else {
			Blockade target;
			Human human;

			human = agent.getMeAsHuman();
			target = getTargetBlockade();
			if (target == null) {
				target = agent.getStuckBlockade(agent.getMeAsHuman(), true);
			}
			if (target == null) {
				if (onTargetPath) {
					return State.GoingToStartToClearPath;
				} else {
					return State.GoingToGoalToClearPath;
				}
			} else {
				double distance;

				this.target = target;

				Point point;

				point = PositionLocate.getPosition(human, getModel());
				distance = DistanceUtilities.getDistanceToBlock(target, point);
				if (distance > maxDistance) {
					return State.GettingCloserToBlock;
				} else {
					if (onTargetPath) {
						return State.ClearingPath;
					} else {
						return State.ClearingBlocksToPath;
					}
				}
			}
		}
	}

	public Blockade getTargetBlockade() {
		Area location;
		Blockade blockade;

		blockade = null;
		location = (Area) agent.getMeAsHuman().getPosition(getModel());
		for (EntityID nextId : location.getNeighbours()) {
			Area area;

			area = getModel().getEntity(nextId, Area.class);
			blockade = getTargetBlockade(area);
			if (blockade != null) {
				return blockade;
			}
		}
		return blockade;
	}

	protected Blockade getTargetBlockade(Area next) {
		Human human;
		PathBlockState state;
		Road road;
		List<Blockade> blockades;
		MultiBlockDistanceComparator comparator;
		Point p1;
		Blockade target;

		human = agent.getMeAsHuman();
		state = RoadUtilities.getPathBlockState(human, next, getModel());
		if (state == PathBlockState.FirstPartIsBlocked) {
			road = (Road) human.getPosition(getModel());
			p1 = PositionLocate.getPosition(human, getModel());
			comparator = new MultiBlockDistanceComparator(getModel(), p1);
		} else if (state == PathBlockState.SecondPartIsBlocked) {
			road = (Road) next;
			p1 = PositionLocate.getPosition(human, getModel());
			comparator = new MultiBlockDistanceComparator(getModel(), p1);
		} else {
			return null;
		}
		blockades = getModel().getBlockades(road);
		if (blockades.isEmpty()) {
			return null;
		}
		Collections.sort(blockades, comparator);
		target = blockades.get(0);
		return target;
	}

	protected Path getPathToStartRoad() {
		Path path;
		Road startRoad = (Road) getModel().getEntity(
				currentClearPathTask.getRoadIdStart());
		path = getSearch().getPath(agent.getMeAsHuman(), startRoad,
				PathType.Shortest);
		if (path != null && !path.isEmpty()) {
			return path;
		}
		return null;
	}

	protected Path getPathToFinishRoad() {
		Path path;
		Road finishRoad = (Road) getModel().getEntity(
				currentClearPathTask.getRoadIdFinish());
		path = getSearch().getPath(agent.getMeAsHuman(), finishRoad,
				PathType.Shortest);
		if (path != null && !path.isEmpty()) {
			return path;
		}
		return null;
	}

	public void addClearPathTask(ClearPath clearPathTask) {
		if (!clearPathTaskQueue.contains(clearPathTask)) {
			clearPathTaskQueue.add(clearPathTask);
		}
	}

	private static final int MINIMUM_MOVE_DISTANCE = 1500;

	private boolean moveFailed() {

		if (state == State.GettingCloserToBlock
				|| state == State.GoingToStartToClearPath
				|| state == State.GoingToGoalToClearPath) {
			double d;
			d = getModel().distance(lastPosition, currentPosition);
			if (d < MINIMUM_MOVE_DISTANCE) {
				return true;
			}
		}
		return false;
	}

	private boolean isOnGoalLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdFinish().getValue();
	}

	private boolean isOnStartLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdStart().getValue();
	}

	protected void addClearPathToRefugeTask() {
		if (agent.existPO) {
			return;
		}
		if (agent.isSelected == false) {
			List<Refuge> refuges = agent.getAllRefuges();
			if (refuges == null) {

				System.err.println(agent.getMeAsHuman().getID() + "No Refuge"
						+ " time" + agent.getTimeStep());
				return;
			}
			for (Refuge next : refuges) {

				if (agent.refugeSort.indexOf(next.getID().getValue()) == agent.PFSort
						.indexOf((agent.getID().getValue()))) {
					ClearPath clearRefugePathTask = new ClearPath(next.getID(),
							agent.location().getID(),
							ClearPathReason.RefugeIsBlocked);
					agent.targetRefuge = next;
					agent.isSelected = true;
					// System.out
					// .println(agent.getID().getValue()
					// + "  is Responsible for "
					// + next.getID().getValue());
					addClearPathTask(clearRefugePathTask);
					break;
				}
			}

		}
	}

	public class IntValueComprator implements Comparator<Object> {

		public int compare(Object arg0, Object arg1) {

			int first = ((Integer) arg0).intValue();
			int second = ((Integer) arg0).intValue();

			if (first > second) {
				return 0;
			} else {
				return 1;
			}
		}
	}
}
