package seu.agent.BasicTask;

import java.awt.Point;
import java.util.ArrayList;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import seu.agent.Police.SEUPoliceForce;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;

public class ClearNearBlockadeTask extends BasicTask<SEUPoliceForce> {

	private Blockade target;
	private final int maxDistance;

	public ClearNearBlockadeTask(SEUPoliceForce agent, int distance) {
		super(agent);
		maxDistance = distance;
	}

	@Override
	public boolean shouldExecute() {
		if ((state == State.Inactive || state == State.ClearingBlock)) {
			State expectedMove;
			expectedMove = getExpectedMove();
			if (expectedMove == State.ClearingBlock) {
				state = State.ClearingBlock;
				return true;
			}
			return false;
		}
		return false;
	}

	protected State getExpectedMove() {
		Area area;
		ArrayList<Building> building = null;
		area = (Area) agent.location();
		target = getPoliceAgent().getTargetBlockade(area, maxDistance);
		if (target == null) {
			building = (ArrayList<Building>) agent.getModel()
					.getBuildings(area);
			if (building != null) {
				for (Building next : building) {
					Road entrance = agent.getEntranceRoad(next);
					if (entrance != null) {
						Blockade blockade = agent
								.getClosestBlockadeInRoad(entrance);
						if (blockade != null) {
							Human human;
							double distance;
							Point agentPoint;
							human = agent.getMeAsHuman();
							agentPoint = PositionLocate.getPosition(human,
									getModel());
							distance = DistanceUtilities.getDistanceToBlock(
									blockade, agentPoint);
							if (distance > maxDistance * 12) {
							} else {
								// System.out.println(agent.getID() + "clear"
								// + blockade);
								target = blockade;
							}
						}
					}
				}
			}
		}
		if (target != null) {
			return State.ClearingBlock;
		}
		return null;
	}

	@Override
	public void execute() {
		if (state == State.ClearingBlock) {
			agent.sendClear(target.getID());
		}
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	protected Area getAreaToClear() {
		return getPoliceAgent().getAreaToClear();
	}

	public SEUPoliceForce getPoliceAgent() {
		return agent;
	}
}
