package seu.agent.BasicTask;

import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import seu.agent.DecisionManager;
import seu.agent.SEUAgent;
import seu.message.MessagePriority;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.object.Exploration;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class ExplorationTask extends
		BasicTask<SEUAgent<? extends StandardEntity>> {

	private Exploration<Building> exploration;
	Path path;

	public ExplorationTask(SEUAgent<? extends StandardEntity> agent) {
		super(agent);
	}

	@Override
	public boolean shouldExecute() {
		agent.sendGetTaskAssigmentMessage();

		if ((state == State.Inactive || state == State.Exploring)
				&& !isExplorationCompeted()) {
			exploration = agent.getExploration();
			List<Building> buildings = exploration.getUnexploredEntities();
			// List<Building> safeBuildings = buildings;
			List<Building> safeBuildings = DecisionManager
					.ignoreBurningBuildings(buildings);
			safeBuildings.removeAll(agent.getTravelMemory()
					.getUnreachableLocations());
			if (safeBuildings.isEmpty()) {
				return false;
			}

			Path path = agent.getSearch().getPath(agent.getMeAsHuman(),
					safeBuildings, PathType.EmptyAndSafe);

			if (path == null) {

				agent.getTravelMemory().addUnreachableLocations(safeBuildings,
						agent.getTimeStep());
				return false;
			} else if (path.hasInfiniteCost()) {
				path = agent.getSearch().getPath(agent.getMeAsHuman(),
						safeBuildings, PathType.Shortest);
				if (path != null && path.isPassable()) {
					ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
							path.getStart(), path.getDestination(),
							ClearPathReason.ExploreBuilding);
					agent.sendMessage(message, MessagePriority.Low);
				}
				return false;
			} else if (path.isPassable()) {
				this.path = path;
				state = State.Exploring;
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	@Override
	public void execute() {
		agent.sendMove(path);
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
		path = null;
	}

	private boolean isExplorationCompeted() {
		exploration = agent.getExploration();
		return exploration.isExplorationCompeted();
	}
}
