package seu.agent.BasicTask;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Edge;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUAgent;
import seu.agent.Firebrigade.SEUFireBrigade;
import seu.object.Building.BurningBuildingComparator;
import seu.utilities.Path;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class ExtinguishFireTask extends BasicTask<SEUFireBrigade> {

	Path path;
	Building building;
	boolean noInferno;

	public ExtinguishFireTask(SEUFireBrigade agent, boolean noInferno) {
		super(agent);
		this.noInferno = noInferno;
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		State state;
		state = getExpectedState();

		if (state == null || state == State.Inactive) {
			state = State.Inactive;
			return false;
		} else {
			this.state = state;
			return true;
		}
	}

	@Override
	public void execute() {
		if (state == State.MovingToFire) {
			agent.sendMove(path);
		} else if (state == State.Extinguishing) {
			agent.sendExtinguish(building);
		}
	}

	public State getExpectedState() {
		List<Building> burningSeenBuildings, extinguishableBuildings, allBurningBuildings;
		Path path;

		// 可见的可以灭火的房屋
		burningSeenBuildings = agent.getBurningBuildingsFromChangeSet(false,
				noInferno);
		extinguishableBuildings = new ArrayList<Building>();
		for (Building burningBuilding : burningSeenBuildings) {
			if (agent.canExtinguish(burningBuilding)) {
				extinguishableBuildings.add(burningBuilding);
			}
		}
		if (!extinguishableBuildings.isEmpty()) {
			Area position;

			position = (Area) agent.getMeAsHuman().getPosition(getModel());
			if (position instanceof Building) {
				Building positionBuilding;

				positionBuilding = (Building) position;
				if (extinguishableBuildings.contains(positionBuilding)) {
					building = positionBuilding;
					return State.Extinguishing;
				}
			}
			BurningBuildingComparator comperator;

			comperator = new BurningBuildingComparator(getModel(),
					agent.getTimeStep());
			Collections.sort(extinguishableBuildings, comperator);
			building = extinguishableBuildings.get(0);
			return State.Extinguishing;
		}
		allBurningBuildings = agent.getBurningBuildings();
		if (allBurningBuildings.isEmpty()) {
			return State.Inactive;
		}
		path = getSearch().getPath(agent.getMeAsHuman(), allBurningBuildings,
				PathType.EmptyAndSafe);
		if (path != null && path.isPassable()) {
			Area destination;
			EntityID previusId;
			Edge edge;

			if (path.size() > 1) {
				destination = path.getDestination(getModel());

				Point target, buildingCenter, position;

				previusId = path.getEntities().get(path.size() - 2);
				edge = destination.getEdgeTo(previusId);

				target = PositionLocate.getCenter(edge);
				buildingCenter = PositionLocate.getCenter(destination);
				position = PositionLocate.getExtensionPoint(buildingCenter,
						target, (int) (5 * SEUAgent.AGENT_RADIUS));
				path.setTarget(position);
			}

			this.path = path;
			return State.MovingToFire;
		} else {
			return State.Inactive;
		}
	}
}
