package seu.agent.BasicTask;

import java.util.Random;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import seu.agent.SEUAgent;
import seu.message.MessagePriority;
import seu.message.Type.AgentIsBuriedMessage;
import seu.message.Type.AgentIsStuckMessage;

public class IsStuckOrBuriedCallHelpTask extends
		BasicTask<SEUAgent<? extends StandardEntity>> {

	public IsStuckOrBuriedCallHelpTask(SEUAgent<? extends StandardEntity> agent) {
		super(agent);
	}

	@Override
	public boolean shouldExecute() {
		if (agent.isBuried()) {
			AgentIsBuriedMessage agentIsBuriedMessage = new AgentIsBuriedMessage(
					agent.location().getID().getValue(), agent.getMeAsHuman()
							.getID().getValue(), agent.getTimeStep(), agent
							.getMeAsHuman().getHP(), agent.getMeAsHuman()
							.getBuriedness(), agent.getMeAsHuman().getDamage());
			agent.sendMessage(agentIsBuriedMessage, MessagePriority.High);

			// 智能体被埋，且被埋的房屋入口处有路障 ，这里发送两条消息
			Area area = (Area) agent.location();

			if (area instanceof Building) {
				Building b = (Building) area;
				if (agent.getEntranceRoad(b) != null) {
					Blockade blockade = agent.getClosestBlockadeInRoad(agent
							.getEntranceRoad(b));

					if (blockade != null) {
						AgentIsStuckMessage agentIsStuckMessage = new AgentIsStuckMessage(
								agent.getEntranceRoad(b).getID());
						agent.sendMessage(agentIsStuckMessage,
								MessagePriority.High);
					}
				}
			}

			state = State.BuriedResting;
			return true;
		} else if (agent.isStuck(agent.getMeAsHuman(), false)) {
			AgentIsStuckMessage agentIsStuckMessage = new AgentIsStuckMessage(
					agent.location().getID());
			agent.sendMessage(agentIsStuckMessage, MessagePriority.High);
			state = State.StuckResting;
			if (agent.getMeAsHuman().getStandardURN() == StandardEntityURN.AMBULANCE_TEAM) {
				Random random;
				int n;
				random = agent.getRandomSource();
				n = random.nextInt(2);
				if (n == 0) {
					return true;
				} else {
					return false;
				}
			}
			return false;
		} else {

			// 智能体被堵在房屋里，但是没有Buriedness
			// 这里仍然需要发送被堵的消息

			return false;
		}
	}

	@Override
	public void execute() {
		agent.sendRest();
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}
}
