package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import seu.agent.SEUAgent;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class RandomWalkTask extends
		BasicTask<SEUAgent<? extends StandardEntity>> {

	private static final int RANDOM_WALK_LENGTH = 8;
	Path path;

	public RandomWalkTask(SEUAgent<? extends StandardEntity> agent) {
		super(agent);
	}

	@Override
	public boolean shouldExecute() {
		if (state == State.Inactive || state == State.ExploringRandom) {
			Path path;

			path = randomWalk();
			if (path.size() > 0) {
				Area randomTarget = (Area) getModel().getEntity(
						path.getDestination());
				path = getSearch().getPath(agent.getMeAsHuman(), randomTarget,
						PathType.Shortest);
				if (path != null && path.isPassable()) {
					this.path = path;
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public void execute() {
		agent.sendMove(path);
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
		path = null;
	}

	protected Path randomWalk() {
		Path path = new Path();
		Set<StandardEntity> seen = new HashSet<StandardEntity>();
		StandardEntity current = agent.location();
		for (int i = 0; i < RANDOM_WALK_LENGTH; ++i) {
			path.add(current.getID());
			seen.add(current);
			List<StandardEntity> neighbours = new ArrayList<StandardEntity>(
					agent.getSearch().findNeighbours(current));
			Collections.shuffle(neighbours, agent.getRandomSource());
			boolean found = false;
			for (StandardEntity next : neighbours) {
				if (seen.contains(next)) {
					continue;
				}
				if (next instanceof Building) {
					continue;
				}
				current = next;
				found = true;
				break;
			}
			if (!found) {
				// We reached a dead-end.
				break;
			}
		}
		return path;
	}

}
