package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Ambulance.SEUAmbulanceTeam;
import seu.message.MessagePriority;
import seu.message.Type.CivilianIsSavedOrDeadMessage;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.utilities.EntityComparator;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class SaveHumanTask extends AmbulanceTeamTask {

	private static final int CIVILIAN_CHECK_PATH_LENGTH = 10;

	private final List<SaveHuman> saveHumanTasks;
	private final List<EntityID> savedHumans;
	private final List<EntityID> pathRequestedPlaces;
	SaveHuman currentTask;
	Human currentHuman;
	Path path;
	boolean isPathRequested;

	public SaveHumanTask(SEUAmbulanceTeam agent) {
		super(agent);
		saveHumanTasks = new ArrayList<SaveHuman>();
		savedHumans = new ArrayList<EntityID>();
		pathRequestedPlaces = new ArrayList<EntityID>();
		isPathRequested = false;
	}

	@Override
	public void taskDone() {
		if (state != State.MovingToRefuge) {
			state = State.Inactive;
		}
	}

	@Override
	public boolean shouldExecute() {
		State expectedMove;
		Path path;

		updateCurrentHuman();
		if (state == State.UnloadingHuman || state == State.Inactive) {
			assignNextHuman();
		}
		if (currentHuman == null) {

			return false;
		}
		expectedMove = getExpectedMove(currentHuman);
		while (expectedMove == State.SavedByOtherAgent
				|| expectedMove == State.HumanIsHelpless) {
			savedHumans.add(currentHuman.getID());
			removeCurrentTask();
			assignNextHuman();

			expectedMove = getExpectedMove(currentHuman);
		}

		if (expectedMove == State.MovingToHuman) {
			Path oppositePath;

			oppositePath = getSearch().getPathFromHuman(currentHuman,
					agent.getMeAsHuman(), PathType.EmptyAndSafe,
					CIVILIAN_CHECK_PATH_LENGTH);
			if (oppositePath != null) {
				if (oppositePath.hasInfiniteCost()) {
					EntityID entityId;

					entityId = currentHuman.getPosition();
					requestPathTo(entityId, ClearPathReason.SaveHuman);
					state = State.Inactive;
					return false;
				}
			} else {
				state = State.Inactive;
				return false;
			}
			path = getSearch().getPath(agent.getMeAsHuman(), currentHuman,
					PathType.EmptyAndSafe);
			if (path != null) {
				if (path.isPassable()) {
					this.path = path;
					state = expectedMove;
					return true;
				} else if (path.hasInfiniteCost()) {
					EntityID entityId;

					entityId = currentHuman.getPosition();
					requestPathTo(entityId, ClearPathReason.SaveHuman);
					state = State.Inactive;
					return false;
				} else {
					state = State.Inactive;
					return false;
				}
			} else {
				state = State.Inactive;
				return false;
			}
		} else if (expectedMove == State.RescuingHuman) {
			state = expectedMove;
			return true;
		} else if (expectedMove == State.LoadingHuman) {
			state = expectedMove;
			CivilianIsSavedOrDeadMessage mesage = new CivilianIsSavedOrDeadMessage(
					currentHuman.getID());
			agent.sendMessage(mesage, MessagePriority.High);

			return true;
		} else if (expectedMove == State.MovingToRefuge) {
			path = agent.getPathToRefuge();
			if (path != null) {
				if (path.isPassable()) {
					this.path = path;
					state = expectedMove;
					return true;
				} else if (path.hasInfiniteCost()) {

					path = agent.getPathToRefuge(PathType.Shortest);
					if (path != null && path.isPassable()) {
						requestPathTo(path.getDestination(),
								ClearPathReason.MoveHumanToRefuge);
						state = State.Inactive;
						return false;
					} else {
						return false;
					}
				} else {
					state = State.Inactive;
					return false;
				}
			} else {
				state = State.Inactive;
				return false;
			}
		} else if (expectedMove == State.UnloadingHuman) {
			state = expectedMove;
			return true;
		} else if (expectedMove == null) {
			if (currentTask != null) {
				removeCurrentTask();
				state = State.Inactive;
				return false;
			} else {
				state = State.Inactive;
				return false;
			}
		} else {
			state = State.Inactive;
			return false;
		}

	}

	@Override
	public void execute() {
		if (state == State.Inactive) {
		}
		if (state == State.MovingToHuman) {
			agent.sendMove(path);
		}
		if (state == State.RescuingHuman) {
			agent.sendRescue(currentHuman);
		}
		if (state == State.LoadingHuman) {
			agent.sendLoad(currentHuman);
		}
		if (state == State.MovingToRefuge) {
			agent.sendMove(path);
		}
		if (state == State.UnloadingHuman) {
			agent.sendUnload();
			savedHumans.add(currentHuman.getID());
			removeCurrentTask();
		}
	}

	protected void assignNextHuman() {
		currentTask = null;
		currentHuman = null;
		if (!saveHumanTasks.isEmpty()) {

			SaveHuman task;
			isPathRequested = false;
			task = getBestTask();
			currentTask = task;
			currentHuman = task.getHuman(getModel());
			currentHuman.setPosition(task.getLocationOfHuman().getID());
			updateCurrentHuman();
		}
	}

	protected State getExpectedMove(Human human) {
		if (isBurning(human)) {
			return State.HumanIsHelpless;
		}
		if (human != null && human.isPositionDefined()) {
			EntityID humanPosition, agentId;

			StandardEntity agentPosition;

			if (human.isHPDefined() && human.getHP() <= 0) {
				return null;
			}
			humanPosition = human.getPosition();
			agentPosition = agent.getMeAsHuman().getPosition(getModel());
			agentId = agent.getID();
			if (agentPosition.getID().equals(humanPosition)) {
				int bury;

				if (!agent.canSee(human.getID())) {
					return State.SavedByOtherAgent;
				}
				bury = human.getBuriedness();
				if (bury > 0) {
					return State.RescuingHuman;
				} else {
					if (agent.isInRefuge()) {
						return State.Inactive;
					} else {
						if (shouldLoadRescuedHuman(human)) {
							return State.LoadingHuman;
						} else {
							return State.SavedByOtherAgent;
						}
					}
				}
			} else if (agentId.equals(humanPosition)) {
				if (agent.isInRefuge()) {
					return State.UnloadingHuman;
				} else {
					return State.MovingToRefuge;
				}
			} else {
				StandardEntity humanLocation;

				humanLocation = getModel().getEntity(humanPosition);
				if (humanLocation instanceof Area
						&& !(humanLocation instanceof Refuge)) {
					return State.MovingToHuman;
				} else {
					return State.SavedByOtherAgent;
				}
			}
		} else {
			return null;
		}
	}

	private void updateCurrentHuman() {
		if (currentHuman != null) {
			EntityID id;
			Human c;

			id = currentHuman.getID();
			c = (Human) getModel().getEntity(id);
			if (c != null) {
				currentHuman = c;
			}
		}
	}

	protected void addSampleTasks() {
		Civilian c1, c2, c3;
		StandardEntity l1, l2, l3;

		c1 = new Civilian(new EntityID(963));
		c1.setPosition(new EntityID(250));
		l1 = c1.getPosition(getModel());
		c2 = new Civilian(new EntityID(964));
		c2.setPosition(new EntityID(253));
		l2 = c2.getPosition(getModel());
		c3 = new Civilian(new EntityID(962));
		c3.setPosition(new EntityID(254));
		l3 = c3.getPosition(getModel());
		saveHumanTasks.add(new SaveHuman(c1, l1, 0));
		saveHumanTasks.add(new SaveHuman(c2, l2, 0));
		saveHumanTasks.add(new SaveHuman(c3, l3, 0));
	}

	private void removeCurrentTask() {
		saveHumanTasks.remove(currentTask);
		currentTask = null;
		currentHuman = null;
	}

	public boolean hasTaskForHuman(Human human) {
		for (SaveHuman task : saveHumanTasks) {
			if (task.getHuman(getModel()).equals(human)) {
				return true;
			}
		}
		return false;
	}

	public void addSaveTask(SaveHuman saveHumanTask) {
		Human human;
		StandardEntity location;

		human = saveHumanTask.getHuman(getModel());
		location = saveHumanTask.getLocationOfHuman();
		if (!hasTaskForHuman(human) && !savedHumans.contains(human.getID())
				&& !(location instanceof Refuge)
				&& (location instanceof Building)) {
			saveHumanTasks.add(saveHumanTask);
			// printSaveHumanTask(saveHumanTask);
		}
	}

	public void removeSaveTask(EntityID humanId) {
		// System.out.println("Cvilian " + humanId.getValue());
		// System.out.println(saveHumanTasks_.toString());
		SaveHuman saveHumanTask = null;
		for (SaveHuman next : saveHumanTasks) {

			if (next.getHuman(getModel()).getID().getValue() == humanId
					.getValue()) {
				saveHumanTask = next;
			}
		}
		if (saveHumanTask != null) {
			if (saveHumanTask.equals(currentTask)) {
				removeCurrentTask();
				state = State.Inactive;
			} else {
				saveHumanTasks.remove(saveHumanTask);
			}
		}
		// System.out.println(saveHumanTasks_.toString());
	}

	public boolean shouldLoadRescuedHuman(Human human) {
		SEUAmbulanceTeam rat;
		List<AmbulanceTeam> nearAgents, saverAgents;

		EntityComparator comparator;
		AmbulanceTeam loader;
		boolean shouldLoad;

		comparator = new EntityComparator();
		rat = getAmbulanceAgent();
		saverAgents = new ArrayList<AmbulanceTeam>();
		nearAgents = rat.getNearbyTeamAgents();
		for (AmbulanceTeam at : nearAgents) {
			if (at.getPosition().equals(human.getPosition())) {
				saverAgents.add(at);
			}
		}
		Collections.sort(saverAgents, comparator);
		loader = saverAgents.get(0);
		shouldLoad = rat.getMeAsHuman().equals(loader);
		return shouldLoad;
	}

	public SaveHuman getBestTask() {
		SaveTaskComparator comparator;

		comparator = new SaveTaskComparator(agent.getMeAsHuman().getLocation(
				getModel()), getModel());
		Collections.sort(saveHumanTasks, comparator);
		return saveHumanTasks.get(0);
	}

	@SuppressWarnings("unused")
	private void requestPathToRefuge() {
		StandardEntity location;
		Path path;
		ClearPathIsNeededMessage message;

		location = agent.location();
		if (pathRequestedPlaces.contains(location.getID())) {
			return;
		}
		path = agent.getPathToRefuge(PathType.EmptyAndSafe);
		if (path == null || !path.isPassable()) {
			path = agent.getPathToRefuge(PathType.Shortest);
			if (path != null && !path.isEmpty() && !isPathRequested) {
				isPathRequested = true;
				message = new ClearPathIsNeededMessage(location.getID(),
						path.getDestination(),
						ClearPathReason.MoveHumanToRefuge);
				agent.sendMessage(message, MessagePriority.High);
				pathRequestedPlaces.add(location.getID());
			}
		}
	}

	private void requestPathTo(EntityID target, ClearPathReason reason) {
		StandardEntity location;
		ClearPathIsNeededMessage message;

		location = agent.location();
		message = new ClearPathIsNeededMessage(location.getID(), target, reason);
		agent.sendMessage(message, MessagePriority.High);
	}

	private boolean isBurning(Human human) {
		if (human != null) {
			StandardEntity position = human.getPosition(getModel());
			boolean onFire = (position instanceof Building)
					&& ((Building) position).isOnFire();
			return onFire;
		}
		return false;
	}
	// private void printSaveHumanTask(SaveHumanTask saveHumanTask){
	// System.out.println("Save human task for human : " +
	// saveHumanTask.getHuman().getID().getValue()) ;
	// System.out.println("Location                  : " +
	// saveHumanTask.getLocationOfHuman().getID().getValue() );
	// System.out.println("Death time                : " +
	// saveHumanTask.getDeathTime()) ;
	//
	// }
}
