package seu.agent.BasicTask;

import java.awt.Point;
import java.util.List;

import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import seu.agent.Police.SEUPoliceForce;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;

public class SaveNearStuckAgentTask extends BasicTask<SEUPoliceForce> {

	private Blockade target;
	private Path path;
	private final int maxDistance;

	public SaveNearStuckAgentTask(SEUPoliceForce agent, int distance) {
		super(agent);
		maxDistance = distance;
	}

	@Override
	public boolean shouldExecute() {
		Road road;
		road = getStuckAgentLocationToClear();
		if (road != null) {
			State expectedMove;
			expectedMove = getExpectedMove(road);
			if (expectedMove == State.ClearingBlock) {
				state = State.ClearingBlock;
				return true;
			} else if (expectedMove == State.MovingToBlockAtCurrentLocation) {
				state = State.MovingToBlockAtCurrentLocation;
				return false;
			}
			return false;

		}
		return false;
	}

	protected State getExpectedMove(Road road) {
		target = getTargetBlockadeToClear(road);

		if (target != null) {
			Point agentPoint;
			agentPoint = PositionLocate.getPosition(agent.getMeAsHuman(),
					getModel());
			if (DistanceUtilities.getDistanceToBlock(target, agentPoint) < maxDistance) {
				return State.ClearingBlock;
			} else {
				path = agent.getPathToBlock(target);
				if (path != null && path.isPassable()) {
					return State.MovingToBlockAtCurrentLocation;
				}
			}

		}
		return null;
	}

	@Override
	public void execute() {
		if (state == State.ClearingBlock) {
			agent.sendClear(target.getID());
		} else if (state == State.MovingToBlockAtCurrentLocation) {
			agent.sendMove(path);
		}
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	protected Road getStuckAgentLocationToClear() {
		List<Human> agents = getPoliceAgent().getFoundAgents();
		for (Human next : agents) {
			// System.out.println(agent.getMeAsHuman() + " I SAW AN AGENT "
			// +next.getID().getValue());
			Road road = agent.getStuckRoad(next, true);
			if (road != null) {
				return road;
			}
		}
		return null;
	}

	public SEUPoliceForce getPoliceAgent() {
		return agent;
	}

	public Blockade getTargetBlockadeToClear(Road road) {
		return getPoliceAgent().getClosestBlockadeInRoad(road);

	}
}
