package seu.agent.BasicTask;

import java.util.Comparator;

import rescuecore2.misc.Pair;
import seu.object.SEUWorldModel;

public class SaveTaskComparator implements Comparator<SaveHuman> {
	private Pair<Integer, Integer> reference;
	private SEUWorldModel world;

	/**
	 * Create a DistanceSorter.
	 * 
	 * @param reference
	 *            The reference point to measure distances from.
	 * @param world
	 *            The world model.
	 */
	public SaveTaskComparator(Pair<Integer, Integer> reference,
			SEUWorldModel world) {
		this.reference = reference;
		this.world = world;
	}

	/**
	 * Compares the standard entites according to distance.
	 * 
	 * @param a
	 *            First StandardEntity to compare
	 * @param b
	 *            Second StandardEntity to compare
	 * @return The difference between distances.
	 */

	@Override
	public int compare(SaveHuman a, SaveHuman b) {
		Pair<Integer, Integer> p1 = a.getHuman(world).getLocation(world);
		Pair<Integer, Integer> p2 = b.getHuman(world).getLocation(world);
		int d1, d2;
		if (p1 == null || p2 == null) {
			d1 = world.distance(reference,
					a.getLocationOfHuman().getLocation(world));
			d2 = world.distance(reference,
					b.getLocationOfHuman().getLocation(world));
		} else {
			d1 = world.distance(reference, p1);
			d2 = world.distance(reference, p2);
		}
		return d1 - d2;
	}
}
