package seu.agent.BasicTask;

import java.util.List;

import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Firebrigade.SEUFireBrigade;
import seu.message.MessagePriority;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class WaterRefillTask extends BasicTask<SEUFireBrigade> {

	Path path;
	int maxWater;
	int minWater;

	public WaterRefillTask(SEUFireBrigade agent, int maxWater, int minWater) {
		super(agent);
		this.maxWater = maxWater;
		this.minWater = minWater;
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		State expectedState;
		expectedState = getExpectedState();
		if (expectedState == State.FillingWater) {
			state = expectedState;
			return true;
		} else if (expectedState == State.MovingToRefuge) {
			path = agent.getPathToRefuge();
			if (path != null && !path.isEmpty()) {
				if (path.isPassable()) {
					state = expectedState;
					return true;
				} else if (path.hasInfiniteCost()) {
					Path path;
					path = agent.getPathToRefuge(PathType.Shortest);
					if (path != null && path.isPassable()) {
						ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
								path.getStart(), path.getDestination(),
								ClearPathReason.SaveMySelf);
						agent.sendMessage(message, MessagePriority.High);
					}
				}
			}
		}
		return false;
	}

	public State getExpectedState() {
		FireBrigade fb;
		int water;

		fb = (FireBrigade) agent.getMeAsHuman();
		water = fb.getWater();
		if (water < maxWater) {
			if (agent.isInRefuge()) {
				if (shouldStayInRefuge()) {
					return State.FillingWater;
				} else {
					return State.Inactive;
				}
			} else {
				if (water <= minWater) {
					return State.MovingToRefuge;
				} else {
					return State.Inactive;
				}
			}
		} else {
			return null;
		}
	}

	@Override
	public void execute() {
		if (state == State.MovingToRefuge) {
			agent.sendMove(path);
		} else if (state == State.FillingWater) {
			agent.sendRest();
		}
	}

	public boolean shouldStayInRefuge() {
		List<FireBrigade> nearAgents;
		List<EntityID> team;
		int teamSize, nearAgentSize;
		double p, r;
		FireBrigade fb;
		int water;

		team = getModel().getTeam();
		teamSize = team.size();
		if (teamSize > 0 && maxWater > 0) {
			fb = (FireBrigade) agent.getMeAsHuman();
			water = fb.getWater();
			nearAgents = getFireAgent().getNearbyTeamAgents();
			nearAgentSize = nearAgents.size() - 1;
			p = ((double) nearAgentSize / teamSize)
					* ((double) water / maxWater);
			p = p * p;
			r = agent.getRandomSource().nextDouble();
			if (r < p) {
				return false;
			}
		}
		return true;
	}

	public SEUFireBrigade getFireAgent() {
		return (SEUFireBrigade) agent;
	}
}
