package seu.agent.Firebrigade;

import java.util.EnumSet;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUCentre;
import seu.message.Type.BuildingIsBurningMessage;
import seu.message.Type.BuildingIsExploredMessage;
import seu.message.Type.BuildingIsExtinguishedMessage;
import seu.message.Type.ExtinguishFireTaskMessage;
import seu.message.Type.Message;
import seu.object.SEUWorldModel;
import seu.object.Partition.FirePartition;

/**
 * SEU centre agent.
 */
public class SEUFireStation extends SEUCentre {

	// private static final Log LOG = LogFactory.getLog(SEUFireStation.class);

	private static final String MAX_WATER_KEY = "fire.tank.maximum";
	private static final String MAX_DISTANCE_KEY = "fire.extinguish.max-distance";
	private static final String MAX_POWER_KEY = "fire.extinguish.max-sum";

	FireTaskManager fireTaskManager;

	public SEUFireStation() {
		super();
	}

	@Override
	public String toString() {
		return "SEU Fire Station";
	}

	@Override
	protected void postConnect() {
		super.postConnect();

		this.world = (SEUWorldModel) super.model;

		world.indexClass(StandardEntityURN.BUILDING, StandardEntityURN.REFUGE);

		config.getIntValue(MAX_WATER_KEY);
		config.getIntValue(MAX_DISTANCE_KEY);
		config.getIntValue(MAX_POWER_KEY);

		fireTaskManager = new FireTaskManager(world);

	}

	@Override
	protected void thinkAndAct() {

		// sendExtinguishFireTasks();

	}

	public void sendExtinguishFireTasks() {
		List<FirePartition> underFireRegions = fireTaskManager
				.getUnderFirePartitions();
		for (FirePartition nextRegion : underFireRegions) {
			List<EntityID> assignedAgents = nextRegion.getAssignedAgents();
			Building tagetBuilding = nextRegion.getTagetBuilding();
			for (EntityID next : assignedAgents) {
				ExtinguishFireTaskMessage message = new ExtinguishFireTaskMessage(
						tagetBuilding.getID().getValue(), next.getValue());
				// System.out.println(" Sending ExtinguishFireTaskMessage message to agent "
				// + next.getValue());
				sendMessage(message, seu.message.MessagePriority.High);
			}

		}
	}

	@Override
	protected void processMessage(Message message) {
		if (message instanceof BuildingIsExploredMessage) {
			BuildingIsExploredMessage explorationMessage = (BuildingIsExploredMessage) message;
			processExplorationMessage(explorationMessage);
		}

		if (message instanceof BuildingIsBurningMessage) {
			BuildingIsBurningMessage buildingIsBurningMessage = (BuildingIsBurningMessage) message;
			processBuildingIsBurningMessage(buildingIsBurningMessage);
		}
		if (message instanceof BuildingIsExtinguishedMessage) {
			BuildingIsExtinguishedMessage buildingIsExtinguishedMessage = (BuildingIsExtinguishedMessage) message;
			processBuildingIsExtinguishedMessage(buildingIsExtinguishedMessage);
		}
	}

	@Override
	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.FIRE_STATION);
	}

}
