package seu.agent.Police;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUAgent;
import seu.agent.BasicTask.BasicTask;
import seu.agent.BasicTask.ClearBlockedRoadTask;
import seu.agent.BasicTask.ClearBlockedRoadToRefugeTask;
import seu.agent.BasicTask.ClearNearBlockadeTask;
import seu.agent.BasicTask.ClearOnBlockadeTask;
import seu.agent.BasicTask.ClearPath;
import seu.agent.BasicTask.ClearPathTask;
import seu.agent.BasicTask.ExplorationTask;
import seu.agent.BasicTask.IsStuckOrBuriedCallHelpTask;
import seu.agent.BasicTask.PartitionExplorationTask;
import seu.agent.BasicTask.RandomWalkTask;
import seu.agent.BasicTask.SaveMyselfTask;
import seu.agent.BasicTask.SaveNearStuckAgentTask;
import seu.message.MessagePriority;
import seu.message.Type.AgentIsStuckMessage;
import seu.message.Type.AssignRefugeMessage;
import seu.message.Type.BuildingIsExploredMessage;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.message.Type.MessageCount;
import seu.message.Type.Message;
import seu.message.Type.MessageType;
import seu.object.SEUWorldModel;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.DistanceComparator;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

/**
 * SEU's Police Office
 */
public class SEUPoliceForce extends SEUAgent<PoliceForce> {

	// 清障距离
	private static final String DISTANCE_KEY = "clear.repair.distance";

	private ClearPathTask clearPathTask;

	private int distance;
	public boolean isDone = false;
	public boolean isSelected = false;
	public boolean existPO = false;
	public int numOfRefuge;
	public int numOfPF;
	public Refuge targetRefuge;
	public ArrayList<StandardEntity> PF = new ArrayList<StandardEntity>();
	public ArrayList<StandardEntity> refuge = new ArrayList<StandardEntity>();
	public ArrayList<Integer> refugeSort = new ArrayList<Integer>();
	public ArrayList<Integer> PFSort = new ArrayList<Integer>();
	public HashMap<Integer, Integer> clearedBlockade = new HashMap<Integer, Integer>();
	public HashMap<Integer, Integer> numOfClear = new HashMap<Integer, Integer>();
	public ArrayList<Integer> PFTarget = new ArrayList<Integer>();

	public SEUPoliceForce() {
		super();

	}

	@Override
	public String toString() {
		return "SEU Police Force # " + getNo();
	}

	@Override
	protected void postConnect() {
		super.postConnect();
		distance = config.getIntValue(DISTANCE_KEY);

	}

	@Override
	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.POLICE_FORCE);
	}

	public List<Road> getBlockedRoads() {
		List<Road> targets = new ArrayList<Road>();
		for (StandardEntity next : world
				.getEntitiesOfType(StandardEntityURN.ROAD)) {
			Road r = (Road) next;
			if (r.isBlockadesDefined() && !r.getBlockades().isEmpty()) {
				targets.add(r);
			}
		}
		Collections.sort(targets, new DistanceComparator(location(), world));
		return targets;
	}

	public List<Blockade> getBlockades() {
		List<Road> roads;
		List<Blockade> blockades;

		roads = world.getEntitiesOfType(Road.class, StandardEntityURN.ROAD);
		blockades = new ArrayList<Blockade>();
		for (Road road : roads) {
			if (road.isBlockadesDefined()) {
				List<EntityID> blockadeIds;

				blockadeIds = road.getBlockades();
				for (EntityID id : blockadeIds) {
					Blockade blockade;

					blockade = world.getEntity(id, Blockade.class);
					blockades.add(blockade);
				}
			}
		}
		return blockades;
	}

	public boolean isBuildingEntrance(EntityID entityId) {
		Area area = (Area) world.getEntity(entityId);
		List<EntityID> neighbors = area.getNeighbours();
		for (EntityID next : neighbors) {
			StandardEntity entity = world.getEntity(next);
			if (entity instanceof Building) {
				// System.out.println("Building entrance : " +
				// entityId.getValue() + " building: " + next.getValue());
				return true;
			}
		}
		return false;
	}

	public Area getAreaToClear() {
		Area area = (Area) location();
		List<EntityID> neighbors = area.getNeighbours();
		for (EntityID next : neighbors) {
			StandardEntity entity = model.getEntity(next);
			if (entity instanceof Road) {
				Area neighborRoad = (Area) entity;
				PathBlockState pbs;

				pbs = RoadUtilities.getPathBlockState(getMeAsHuman(),
						neighborRoad, world);
				if (pbs == PathBlockState.FirstPartIsBlocked) {
					return area;
				} else if (pbs == PathBlockState.SecondPartIsBlocked) {
					return neighborRoad;
				}
			}
		}
		return null;
	}

	public Blockade getTargetBlockade(Area area, int maxDistance) {
		if (!area.isBlockadesDefined()) {
			return null;
		}
		List<EntityID> ids = area.getBlockades();
		Point agentPoint;
		// Find the first blockade that is in range.

		agentPoint = PositionLocate.getPosition(getMeAsHuman(), world);
		// int x = me().getX();
		// int y = me().getY();
		for (EntityID next : ids) {
			Blockade b = (Blockade) model.getEntity(next);
			double d;

			d = DistanceUtilities.getDistanceToBlock(b, agentPoint);
			if (maxDistance < 0 || d < maxDistance) {
				return b;
			}
		}
		return null;
	}

	public Path getTargetBlockadeAtLocation() {

		return null;
	}

	protected void processMessage(MessageCount message) {
		int counter = message.getCounter();
		MessageCount reply = new MessageCount(counter + 1);
		sendMessage(reply, MessagePriority.High);
	}

	@Override
	protected void thinkAndAct() {
	}

	public boolean isOnBlockedRoad() {
		StandardEntity location = location();
		if (location instanceof Road) {
			Road r = (Road) location;
			if (r.isBlockadesDefined() && r.getBlockades().size() > 0) {
				return true;
			}
		}
		return false;
	}

	@Override
	protected void processMessage(Message message) {
		if (message instanceof BuildingIsExploredMessage) {
			BuildingIsExploredMessage explorationMessage = (BuildingIsExploredMessage) message;
			processExplorationMessage(explorationMessage);
		}
		if (message instanceof ClearPathIsNeededMessage) {
			ClearPathIsNeededMessage clearPathMessage = (ClearPathIsNeededMessage) message;
			processClearPathMessage(clearPathMessage);
		}
		if (message instanceof AgentIsStuckMessage) {
			AgentIsStuckMessage agentIsStuckMessage = (AgentIsStuckMessage) message;
			processAgentIsStuckMessage(agentIsStuckMessage);
		}
		if (message instanceof AssignRefugeMessage) {
			AssignRefugeMessage assignRefugeMessage = (AssignRefugeMessage) message;
			processAssignRefugeMessage(assignRefugeMessage);
		}
	}

	protected void processAssignRefugeMessage(AssignRefugeMessage message) {
		EntityID agentID = message.getAgentId();
		EntityID targetRefugeID = message.getRefugeId();

		if (agentID.getValue() == this.getID().getValue()) {
			Refuge refuge = (Refuge) world.getEntity(targetRefugeID);
			this.targetRefuge = refuge;
			// System.out.println(this.getID().getValue() + "is assigned refuge"
			// + refuge.getID().getValue());
		}
	}

	protected void processClearPathMessage(ClearPathIsNeededMessage message) {
		EntityID startId = message.getStartId();
		StandardEntity startRoad = world.getEntity(startId);
		EntityID finishId = message.getFinishId();
		StandardEntity finish = world.getEntity(finishId);
		if (finish instanceof Building) {
			Road entrance = getEntranceRoad((Building) finish);
			if (entrance != null) {
				finishId = entrance.getID();
			}
		}

		if (!PFTarget.isEmpty() && PFTarget.contains(finishId.getValue())) {

			// System.out.println("Same target!");
			return;
		}
		// 如果该路径在自己负责的分区内
		if (isInMyPartition(startRoad)) {
			PFTarget.add(finishId.getValue());
			ClearPath task = new ClearPath(startId, finishId,
					message.getClearPathReason());
			clearPathTask.addClearPathTask(task);
		}
	}

	// 智能体被堵，要及时响应
	@SuppressWarnings("unused")
	protected void processAgentIsStuckMessage(AgentIsStuckMessage message) {
		EntityID startId = message.getLocationId();
		EntityID finishId = message.getLocationId();
		StandardEntity startRoad = world.getEntity(startId);
		int dis = world.getDistance(this.getID(), startId);
		if (!PFTarget.isEmpty() && PFTarget.contains(finishId.getValue())) {
			return;
		}
		// 警察相应智能体的最远距离
		if (dis <= 75000) {

			ClearPath task = new ClearPath(startId, finishId,
					ClearPathReason.AgentIsStuck);
			clearPathTask.addClearPathTask(task);
			PFTarget.add(finishId.getValue());

		} else {
		}
	}

	@Override
	protected boolean canRescue() {
		return false;
	}

	@SuppressWarnings({ "rawtypes" })
	@Override
	public List<BasicTask> getAgentTasks() {
		if (!world.getEntitiesOfType(StandardEntityURN.POLICE_OFFICE).isEmpty()) {
			existPO = true;
		}
		List<BasicTask> tasks;
		tasks = new ArrayList<BasicTask>();

		tasks.add(new IsStuckOrBuriedCallHelpTask(this));
		tasks.add(new ClearOnBlockadeTask(this, distance));
		tasks.add(new SaveMyselfTask(this));
		// 清理路障，保证能通过
		// 这两个顺序还要考虑
		// 在考虑救智能体的问题上，这里以后将常识用中心调配的方法实现
		tasks.add(new ClearBlockedRoadToRefugeTask(this, distance));
		tasks.add(new SaveNearStuckAgentTask(this, distance));
		tasks.add(new ClearNearBlockadeTask(this, distance / 12));

		clearPathTask = new ClearPathTask(this, distance);
		tasks.add(clearPathTask);

		tasks.add(new ClearBlockedRoadTask(this, distance));
		tasks.add(new ExplorationTask(this));
		tasks.add(new PartitionExplorationTask(this));
		tasks.add(new RandomWalkTask(this));
		return tasks;
	}

	public Blockade getClosestBlockade() {
		Human human;
		List<Blockade> blockades;
		MultiBlockDistanceComparator comparator;
		Point position;
		Blockade target;

		human = getMeAsHuman();
		position = PositionLocate.getPosition(human, world);
		comparator = new MultiBlockDistanceComparator(getModel(), position);
		blockades = getBlockades();

		if (blockades.isEmpty()) {
			return null;
		}
		Collections.sort(blockades, comparator);

		target = blockades.get(0);
		return target;
	}

	public Blockade getClosestBlockadeInCurrentLocation() {
		Human human;
		Area area;
		Road road;
		Blockade target;

		human = getMeAsHuman();
		area = (Area) human.getPosition(getModel());

		if (area instanceof Road) {
			road = (Road) area;
			target = getClosestBlockadeInRoad(road);
		} else {
			target = null;
		}

		return target;
	}

	public Path getPathToClosestBlockedRoad() {
		Path path;
		List<Road> blockedRoads;
		Human human;

		human = getMeAsHuman();
		blockedRoads = getBlockedRoads();
		path = search.getPath(human, blockedRoads, PathType.Shortest);
		return path;
	}

	@Override
	public List<MessageType> getMessagesToListen() {
		List<MessageType> types;
		types = new ArrayList<MessageType>();
		types.add(MessageType.ClearPathIsNeededMessage);
		types.add(MessageType.AgentIsStuckMessage);
		types.add(MessageType.AgentIsBuriedMessage);
		types.add(MessageType.BuildingIsExploredMessage);
		types.add(MessageType.BuildingIsBurningMessage);
		types.add(MessageType.BuildingIsExploredMessage);
		types.add(MessageType.AssignRefugeMessage);
		return types;
	}

	public SEUWorldModel getWorldModel() {
		return world;
	}

	@Override
	public void sendClear(EntityID targetId) {
		Blockade blockade;
		int id = targetId.getValue();
		blockade = world.getEntity(targetId, Blockade.class);
		if (blockade == null) {
			sendMove(timeStep, randomWalk());
			return;

		}
		if (clearedBlockade.containsKey(id)) {
			if (clearedBlockade.get(id) == blockade.getRepairCost()) {
				int num = numOfClear.get(id);
				num++;
				numOfClear.put(id, num);
				if (num > 5) {
					// System.out.println(this.getID().getValue() + "blockade"
					// + id + " is null I'm randmom walk");
					sendMove(timeStep, randomWalk());
					return;
				}

			}
		}
		if (!RoadUtilities.doesBlockExists(blockade, world)) {
			sendMove(timeStep, randomWalk());
			return;
		}
		Point point = PositionLocate.getPosition(this.getMeAsHuman(),
				getModel());
		int dis = DistanceUtilities.getDistanceToBlock(blockade, point);
		if (dis > distance) {
			sendMove(timeStep, randomWalk());
			return;
		}
		super.sendClear(targetId);
		if (!clearedBlockade.containsKey(targetId.getValue())) {
			clearedBlockade.put(id, blockade.getRepairCost());
			numOfClear.put(id, 1);
		}
		if (clearedBlockade.containsKey(targetId.getValue())) {
			if (clearedBlockade.get(id) != blockade.getRepairCost()) {
				clearedBlockade.put(id, blockade.getRepairCost());
				numOfClear.put(id, 1);
			}
		}
	}
}