package seu.agent.Police;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import rescuecore2.messages.Command;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUCentre;
import seu.agent.Path.ClearPathTaskManager;
import seu.message.Type.AgentIsBuriedMessage;
import seu.message.Type.AssignRefugeMessage;
import seu.message.Type.ClearPathIsAssignedMessage;
import seu.message.Type.Message;
import seu.message.Type.MessageType;
import seu.object.SEUWorldModel;

public class SEUPoliceOffice extends SEUCentre {

	private ClearPathTaskManager clearPathTaskManager;
	private ArrayList<StandardEntity> refuge = new ArrayList<StandardEntity>();
	private ArrayList<StandardEntity> pf = new ArrayList<StandardEntity>();
	private HashSet<StandardEntity> assignedPF = new HashSet<StandardEntity>();
	private boolean isAssigned = false;

	public SEUPoliceOffice() {
		super();
	}

	@Override
	public String toString() {
		return "SEU Police Office";
	}

	@Override
	protected void postConnect() {
		super.postConnect();

		this.world = (SEUWorldModel) super.model;

		world.indexClass(StandardEntityURN.BUILDING, StandardEntityURN.REFUGE);

		clearPathTaskManager = new ClearPathTaskManager(world);

	}

	@Override
	protected void think(int time, ChangeSet changed, Collection<Command> heard) {
		if (time == startTime) {
			int[] channels;
			setCommunicationCondition();
			channels = getChannelsToSubscribe();
			sendSubscribe(time, channels);
		}
		if (time >= startTime) {
			processHearing(heard);
		}
		setTimeStep(time);
		if (time > startTime + 2) {
			if (!isAssigned) {
				AssignRefuge();
			}
			sendMessagesFromQueue();
		}
	}

	public void AssignRefuge() {
		isAssigned = true;
		HashSet<StandardEntity> temp;
		temp = (HashSet<StandardEntity>) world
				.getEntitiesOfType(StandardEntityURN.REFUGE);
		Iterator<StandardEntity> it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = (StandardEntity) it.next();
			refuge.add(t);
		}
		temp = (HashSet<StandardEntity>) world
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE);
		it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = (StandardEntity) it.next();
			pf.add(t);
		}
		for (StandardEntity r : refuge) {

			StandardEntity targetPF = pf.get(0);
			int dis = world.getDistance(r.getID(), pf.get(0).getID());
			for (StandardEntity p : pf) {
				Human man = (Human) p;
				if (man.isBuriednessDefined()) {
					continue;
				}
				if (assignedPF.contains(p))
					continue;
				if (dis > world.getDistance(r.getID(), p.getID())) {
					targetPF = p;
					dis = world.getDistance(r.getID(), p.getID());

				}
			}
			if (targetPF != null && !assignedPF.contains(targetPF)) {
				AssignRefugeMessage message = new AssignRefugeMessage(
						targetPF.getID(), r.getID());
				// System.out.println(timeStep + "Send" + message);
				assignedPF.add(targetPF);
				sendMessage(message, seu.message.MessagePriority.High);
			}
		}
	}

	@Override
	protected void thinkAndAct() {

		// if (getTimeStep() % 7 == 0 && getTimeStep() > 3) {
		clearPathTaskManager.updateState(world, getTimeStep());
		sendClearPathTasks();
		// fireTaskManager.printFireRegions();
		// }
	}

	public void sendClearPathTasks() {
		List<PoliceAgentTask> policeForceTasks = clearPathTaskManager
				.getPoliceForceTasks();
		for (PoliceAgentTask nextPolice : policeForceTasks) {
			ClearPathIsAssignedMessage message = new ClearPathIsAssignedMessage(
					nextPolice.getClearPathTask().getRoadIdStart(), nextPolice
							.getClearPathTask().getRoadIdFinish(),
					nextPolice.getID());
			sendMessage(message, seu.message.MessagePriority.High);
		}
	}

	@Override
	protected void processMessage(Message message) {
		if (message instanceof AgentIsBuriedMessage) {

			AgentIsBuriedMessage agentIsBuriedMessage = (AgentIsBuriedMessage) message;

			processAgentIsBuriedMessage(agentIsBuriedMessage);
		}
	}

	protected void processAgentIsBuriedMessage(AgentIsBuriedMessage message) {
		EntityID id = new EntityID(message.getAgentId());
		StandardEntity entity = world.getEntity(id);
		Human man = (Human) entity;
		try {
			man.setBuriedness(message.getBuriedness());
		} catch (Exception e) {
			System.err.println("Messeage Get Buriedness Error");
		}
	}

	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.POLICE_OFFICE);
	}

	@Override
	public List<MessageType> getMessagesToListen() {
		List<MessageType> types;

		types = new ArrayList<MessageType>();
		types.add(MessageType.AgentIsStuckMessage);
		types.add(MessageType.AgentIsBuriedMessage);
		types.add(MessageType.BuildingIsExploredMessage);
		types.add(MessageType.BuildingIsBurningMessage);
		types.add(MessageType.BuildingIsExploredMessage);
		return types;
	}
}
