package seu.object.Building;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.standard.entities.AmbulanceCentre;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.FireStation;
import rescuecore2.standard.entities.PoliceOffice;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import seu.object.SEUWorldModel;

public class BuildingInfo {

	private Building building;

	// 得到当前房屋
	public Building getBuilding() {
		return building;
	}

	private List<Building> negihbours;

	public static final int MAX_DIST_NEAR_BUILDINGS = 7000;
	private static final int MIN_HEATING_VALUE = 0;

	int latestTemperature;
	int temperatureUpdateTime;

	public BuildingInfo(Building building) {

		this.building = building;

		if (building.isTemperatureDefined()) {
			latestTemperature = building.getTemperature();
		} else {
			latestTemperature = 0;
		}
	}

	// 得到危险的房屋
	// 这里是根据范围来限定的，房屋周围7米，如果有必要，可以扩大距离
	public ArrayList<Building> inDangerNeighbors() {

		if (building.getFieryness() == 4 || building.getFieryness() == 0
				|| !building.isFierynessDefined()) {
			return null;
		}

		// 首先要计算该房屋的连接房屋
		// 合理的方式是使用得到房屋辐射到的第一个房屋
		// 但是这样处理的话必然要构建每个房屋的墙，需要根据房屋的Apex List构建房屋的每条线
		// 所以这里简单的用范围限定

		ArrayList<Building> ret = new ArrayList<Building>();

		if (negihbours == null)
			System.err.println("The Building Have No Neighbours");

		for (Building b : negihbours) {
			if (b.getFieryness() == 0) {
				if (!(b instanceof Refuge) && !(b instanceof AmbulanceCentre)
						&& !(b instanceof PoliceOffice)
						&& !(b instanceof FireStation)) {
					ret.add(b);
				}
			} else if (b.getFieryness() >= 4 && b.getFieryness() <= 7) {
				ret.add(b);
			}
		}
		return ret;

	}

	// 判断房屋是否正在燃烧
	public boolean isBurning() {
		// 如果该房屋的燃烧度没有定义，假定为没有燃烧
		if (!building.isFierynessDefined())
			return false;
		return (1 <= building.getFieryness() && building.getFieryness() <= 3);
	}

	// 给房屋周围限定距离
	public List<Building> getNeighbours(SEUWorldModel world) {
		if (negihbours == null) {
			negihbours = new ArrayList<Building>();
			negihbours = world.findNearBuildings(building,
					MAX_DIST_NEAR_BUILDINGS);
			return negihbours;
		}
		return negihbours;
	}

	// 判断给定房屋是否是该房屋的周围房屋
	public boolean isNeighbourBuilding(Building building, SEUWorldModel world) {
		if (negihbours == null) {
			negihbours = getNeighbours(world);
		}
		if (!negihbours.isEmpty()) {
			for (Building neighbour : negihbours) {
				if (building.getID().getValue() == neighbour.getID().getValue()) {
					return true;
				}
			}
		}
		return false;
	}

	public int getLatestTemperature() {
		return latestTemperature;
	}

	public void setLatestTemperature(int latestTemperature) {
		this.latestTemperature = latestTemperature;
	}

	public boolean isHeating() {
		if (building.isFierynessDefined()
				&& building.getFierynessEnum() != Fieryness.BURNT_OUT) {
			int d;
			boolean heating;

			d = getTemperatureDifference();
			heating = (d > MIN_HEATING_VALUE);
			return heating;
		} else {
			return false;
		}
	}

	public double getHeatingSpeed(int timeStep) {
		int timeDifference;

		timeDifference = timeStep - temperatureUpdateTime;
		if (timeDifference > 0) {
			int temperatureDifference;
			double r;

			temperatureDifference = getCurrentTemperature() - latestTemperature;
			r = (double) temperatureDifference / timeDifference;
			return r;
		} else {
			return 0;
		}
	}

	public int getCurrentTemperature() {
		if (building.isTemperatureDefined()) {
			return building.getTemperature();
		} else {
			return 0;
		}
	}

	public int getTemperatureDifference() {
		int temperature, difference;

		temperature = getCurrentTemperature();
		difference = temperature - latestTemperature;
		return difference;
	}

	public void updateTemperature(int timeStep) {
		if (building.isTemperatureDefined()) {
			latestTemperature = building.getTemperature();
			temperatureUpdateTime = timeStep;
		}
	}
}
