/**
 * 将着火区域进行划分，着火房屋放入火区
 */
package seu.utilities;

import java.util.ArrayList;
import java.util.HashSet;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardWorldModel;
import seu.object.SEUWorldModel;
import seu.object.Building.BuildingInfo;

/**
 * @author knight
 * 
 */
public class FireSite {

	public final SEUWorldModel world;

	// 火区内所有着火房屋
	private final HashSet<Building> fires = new HashSet<Building>();

	// 即将加入火区的房屋
	private final HashSet<Building> dangerousFires = new HashSet<Building>();

	// 火区周围的房屋
	private final HashSet<Building> siteNighbors = new HashSet<Building>();

	// 火区的两个参数，世界模型和当前房屋
	public FireSite(BuildingInfo fire, SEUWorldModel world) {
		super();

		fires.add(fire.getBuilding());

		this.world = world;

		ArrayList<Building> inDangerNeighbors = fire.inDangerNeighbors();

		if (inDangerNeighbors != null && inDangerNeighbors.size() > 0) {
			dangerousFires.add(fire.getBuilding());
		}

		// 加入周围没有着火的房屋，这个范围需要调整
		siteNighbors.addAll(fire.getNeighbours(world));

	}

	/**
	 * 检查着火的房屋是否可以加入火区
	 * 
	 * @return 1, 已经存在, 0 if can, -1 不可以
	 */
	public int addFire(BuildingInfo fire) {
		if (containsFire(fire.getBuilding())) {
			return 1;
		}
		if (!isSiteNeighbor(fire.getBuilding())) {
			return -1;
		}
		fires.add(fire.getBuilding());

		siteNighbors.remove(fire.getBuilding());

		for (Building b : fire.getNeighbours(world)) {
			if (!containsFire(b)) {
				siteNighbors.add(b);
			}
		}

		updateDangerousFires();

		return 0;
	}

	// 根据世界模型更新火区
	// 这里不知道Process Hearing更新的房屋信息是否也并入了自己的世界模型，需要测试
	public void updateSite(StandardWorldModel world) {

	}

	// 更新危险的房屋
	private void updateDangerousFires() {
		// TODO 更新危险的房屋，这里首先得到所有的房屋

	}

	// 如果该火区内包含该房屋
	public boolean containsFire(Building fire) {
		return fires.contains(fire);
	}

	public boolean isSiteNeighbor(Building b) {
		return siteNighbors.contains(b);
	}

}
